/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
package jdct;

import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class JdctInstanceGui extends Frame
{
	//Font defaultFont = new Font("monospaced",Font.PLAIN,10);
	Font defaultFont = new Font("monospaced",Font.PLAIN,12);
	//Font defaultFont = new Font("default",Font.PLAIN,10);
	//Font defaultFont;

	private static final int TEXT_UPDATE_LENGTH = 40;
	private static final int DEFAULT_NAME_LENGTH = 5;
	private static final int DEFAULT_TYPE_LENGTH = 8;
	private static final int DEFAULT_DESC_LENGTH = 28;
	private static final int SCREENWIDTH = 650;
	private static final int SCREENHEIGHT = 400;

	private int nameOffset = 10;
	// offset in pixels
	    private int typeOffset = 0;
	private int descOffset = 0;
	private int dataOffset = 0;
	private int baseline;
	private int fontHeight;

	private Vector nameText = new Vector();
	private Vector typeText = new Vector();
	private Vector descText = new Vector();
	private Vector dataEntry = new Vector();

	private int flip = 0;

	private JdctRecordInstance recordInstance;

	private ScrollPane scrollPane;
	private Panel paneln;
	private Panel panelw;
	private Panel panel;
	private Panel dataEntryPanel;
	private TextCanvas textCanvas;

	private Panel topPanel;
	private Label instanceLabel;
	private Button closeButton;

	private int numFields = 0;
	private static int offsetX = 100;
	private static int offsetY = 205;


	public JdctInstanceGui(JdctRecordInstance recordInstanceIn) {
		super();

		//this.setBackground(SystemColor.control);
		this.setBackground(Color.lightGray);

		recordInstance = recordInstanceIn;

		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				JdctInstanceGui.this.dispose();
				    recordInstance.removeRecordInstance();
			}
		}
		);

		topPanel = new Panel(new BorderLayout(0,0));
		this.add(topPanel);

		instanceLabel = new Label("No instance loaded");
		topPanel.add("North",instanceLabel);

		Panel closePanel = new Panel(new FlowLayout(FlowLayout.CENTER,0,2));
		closeButton = new Button("Close");
		closePanel.add("South",closeButton);
		topPanel.add("South",closePanel);
		closeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JdctInstanceGui.this.setVisible(false);
				recordInstance.removeRecordInstance();
			} 
		}
		);



		scrollPane = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
		topPanel.add("Center",scrollPane);

		paneln = new Panel(new BorderLayout(0,0));
		scrollPane.add(paneln);

		panelw = new Panel(new BorderLayout(0,0));
		paneln.add("North",panelw);

		panel = new Panel(new GridLayout(1,2));
		panelw.add("West",panel);

		textCanvas = new TextCanvas();
		panel.add(textCanvas);

		dataEntryPanel = new Panel(new GridLayout(0,1));
		panel.add(dataEntryPanel);

		setup();

		//this.setTitle("No instance loaded");
		this.pack();
		this.setSize(SCREENWIDTH,SCREENHEIGHT);
		offsetX += 50;
		offsetY += 50;
		this.setLocation(offsetX,offsetY);
		this.setVisible(true);
	}

	public JdctRecordInstance getRecordInstance(){
		return recordInstance;
	}

	void setup() {
		this.addNotify();
		this.setupFontData();
	}

	private String getDefaultString(int length)
	    {
		StringBuffer buff = new StringBuffer(length);
		for (int i=0;i<length; i++) buff.insert(i,'W');
		return(new String(buff));
	}

	void setupFontData() {
		//defaultFont = this.getFont();
		this.setFont(defaultFont);
		Toolkit toolkit = this.getToolkit();
		FontMetrics fm = toolkit.getFontMetrics(defaultFont);

		baseline = fm.getLeading() + fm.getAscent();
		fontHeight = fm.getHeight();

		typeOffset = nameOffset +
		    fm.stringWidth(getDefaultString(DEFAULT_NAME_LENGTH + 1));
		descOffset = typeOffset +
		    fm.stringWidth(getDefaultString(DEFAULT_TYPE_LENGTH + 1));
		dataOffset = descOffset +
		    fm.stringWidth(getDefaultString(DEFAULT_DESC_LENGTH + 1));
	}

	public void setNFields(int num) {
		// remove field data from previous record
		    nameText.removeAllElements();
		typeText.removeAllElements();
		descText.removeAllElements();
		dataEntry.removeAllElements();
		numFields = num;

		scrollPane.setScrollPosition(0,0);

		Component[] component = dataEntryPanel.getComponents();
		for (int i=0;i<dataEntryPanel.getComponentCount(); i++) {
			if (component[i] instanceof JdctLinkFieldInstance)
				((JdctLinkFieldInstance)component[i]).removeDialog();
		}
		dataEntryPanel.removeAll();

		textCanvas.setSize(0,0);
		this.pack();
	}

	public void setFieldName(int i,String name) {
		nameText.addElement((Object)name);
	}

	public void setFieldType(int i,String type) {
		typeText.addElement((Object)type);
	}

	public void setFieldDesc(int i,String desc) {
		descText.addElement((Object)desc);
	}

	public JdctLinkFieldInstance addLinkField(JdctFrame frame,
	    String recordName, String fieldName, String text,int linkType) {
		JdctLinkFieldInstance linkField = new JdctLinkFieldInstance(frame,
		    this,recordName,fieldName,text,linkType);
		dataEntry.addElement((Object)linkField);
		dataEntryPanel.add((Component)linkField);
		return linkField;
	}

	public void addTextField(JdctFrame frame,
	    String recordName, String fieldName, String text) {
		JdctTextFieldInstance textField = new JdctTextFieldInstance(frame,
		    this,recordName,fieldName,text);
		textField.setColumns(TEXT_UPDATE_LENGTH);
		dataEntry.addElement((Object)textField);
		dataEntryPanel.add((Component)textField);
	}

	public void addMenuField(JdctFrame frame,
	    String recordName, String fieldName, String[] menu,String selection) {
		JdctMenuFieldInstance menuField = new JdctMenuFieldInstance(frame,
		    this,recordName,fieldName,menu,selection);
		dataEntry.addElement((Object)menuField);
		dataEntryPanel.add((Component)menuField);
	}

	public JdctMenuFormFieldInstance addMenuFormField(JdctFrame frame,
	    String recordName, String fieldName, String[] menu,String selection) {
		JdctMenuFormFieldInstance menuFormField = 
		    new JdctMenuFormFieldInstance(frame,this,recordName,fieldName,menu,selection);
		dataEntry.addElement((Object)menuFormField);
		dataEntryPanel.add((Component)menuFormField);
		return menuFormField;
	}

	public void setInstanceLabel(String labelIn) {
		instanceLabel.setText(labelIn);
	}

	MessageDialog messageDialog(String title,String message,Point location)
	    {
		return new MessageDialog(title,message,location);
	}

	class TextCanvas extends Canvas
	    {
		public void paint(Graphics g) {
			if (nameText.isEmpty()) return;
			if (nameText.size() != numFields) {
				new MessageDialog("ERROR",
				    "Incorrect number of fields="+ numFields +
				    "nameText.size()="+nameText.size(),new Point(100,100));
				return;
			}
			Component[] component = dataEntryPanel.getComponents();
			if (numFields>0) scrollPane.getVAdjustable().setUnitIncrement(component[1].getLocation().y);
			for(int i = 0;(i < numFields);++i) {
				int y = component[i].getLocation().y+fontHeight;
				g.drawString((String) nameText.elementAt(i), nameOffset ,y);
				g.drawString((String) typeText.elementAt(i), typeOffset ,y);
				g.drawString((String) descText.elementAt(i), descOffset ,y);
			}
		}
	}

	class MessageDialog extends Dialog
	    {
		public MessageDialog(String title,String message,Point location)
		    {
			super(JdctInstanceGui.this,title,true);

			this.setLayout(new BorderLayout(15, 15));
			this.add("Center", new Label(message));

			Panel panel = new Panel();
			panel.setLayout(new FlowLayout(FlowLayout.CENTER, 25, 15));

			this.add("South", panel);

			Button okButton = new Button("Ok");
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					MessageDialog.this.dispose();
				}
			}
			);

			this.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					((Dialog)e.getSource()).dispose();
				}
			}
			);

			panel.add(okButton);

			this.pack();
			this.setLocation(location);
			this.setVisible(true);
		}
	}
}

