/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
package jdct;

import jdct.*;
import dbStatic.*;
import java.lang.* ;
import java.awt.* ;
import java.awt.event.*;
import java.io.*;

class JdctManager implements ActionListener, ItemListener, WindowListener
{
	private JdctFrame frame;
	private JdctMenu menu;
	private JdctSelectRecord selectRecord;
	private String currentFilename;

	private static DbBase dbBase = new DbBase();

	JdctManager(String[] filename)
	    {
		//frame = new JdctFrame(this);
		frame = new JdctFrame(filename);
		//menu = new JdctMenu(frame,this);
		menu = new JdctMenu(frame);
		frame.setMenuBar(menu);
		selectRecord = new JdctSelectRecord(frame,this);

		for (int i=0; i<filename.length; i++) {
			if (filename[i].toLowerCase().endsWith(".db"))
				openDbFile(filename[i]);
			else openDbdFile(filename[i]);
		}
		frame.setLocation(25,20);
		frame.pack();
		//frame.show();
	}

	public void actionPerformed(ActionEvent e)
	    {
		String selection = e.getActionCommand();

		if(selection.equals("Get instances")) {
			getRecordInstanceList();

		} else if ("Open dbd".equals(selection)) {
			String fn = menu.jdctDbdFileDialog(frame,
			    "Enter dbd filename",
			    FileDialog.LOAD);
			openDbdFile(fn);
			//jdctMenu.getMenu("File").getItem("Save").setEnabled(true);
		} else if ("Open db".equals(selection)) {
			String fn = menu.jdctFileDialog(frame,
			    "Enter db filename", FileDialog.LOAD);
			openDbFile(fn);
			//jdctMenu.getMenu("File").getItem("Save").setEnabled(true);
		} else if ("Clear db".equals(selection)) {
			cleardb();
			frame.getInstanceList().removeAll();
			frame.clearAllRecordInstances();

		} else if ("Clear".equals(selection)) {
			clear();
			frame.getInstanceList().removeAll();
			frame.clearAllRecordInstances();
			//getRecordInstance().loadInstance(null);

		} else if ("Save".equals(selection)) {
			//String fn = menu.jdctFileDialog(frame,
			    //    "Enter db filename",
			    //    FileDialog.SAVE);
			saveFile(currentFilename);

		} else if ("Save as".equals(selection)) {
			String fn = menu.jdctFileDialog(frame,
			    "Enter db filename",
			    FileDialog.SAVE);
			saveAsFile(fn);

		} else if ("Create".equals(selection)) {
			String type = menu.getCreateRecordType();
			String name = menu.getCreateName();
			if (name!=null && name.length()>0) {
				createRecord(name,type);
			}
		} else if ("Delete".equals(selection)) {
			String name = menu.getRecordName();
			if (name!=null) {
				deleteRecord(name);
			}
		} else if ("Copy".equals(selection)) {
			String name = menu.getRecordName();
			String newName = menu.getCopyName();
			if (newName!=null && newName.length()>0) {
				copyRecord(name,newName);
			}
		} else if ("Rename".equals(selection)) {
			String name = menu.getRecordName();
			String newName = menu.getRenameName();
			if (newName!=null && newName.length()>0) {
				renameRecord(name,newName);
			}
		} else if ("New Window".equals(selection)) {
			frame.addRecordInstance();
		} else if ("Print".equals(selection)) {
			printFile();
		} else if ("Quit".equals(selection)) {
			new QuitDialog(frame,"Quit Dialog","Quit Jdct?");
		} else if ("Help".equals(selection)) {
		}
	}

	public void itemStateChanged(ItemEvent e)
	    {
		if (e.getSource() instanceof List ) {
			String name = frame.getRecordName();
			if ( name != null ) {
				menu.setRecordName(name);
				getRecordInstance().loadInstance(name);
			}
		}
	}

	//class MessageDialog extends Dialog
	    //    {
		//	}

	final public void cleardb()
	    {
		frame.setFilename("Db filename: No filename");
		currentFilename = null;
		selectRecord.newInputFileNotify();
		menu.setFilename(null);

		DbBase dbBase = this.getDbBase();
		DbEntry dbEntry = new DbEntry(dbBase);
		if(!dbEntry.firstRecordType()) return;
		do {
			while (dbEntry.firstRecord()) {
				dbEntry.deleteRecord();
			}
		} while (dbEntry.nextRecordType());
	}

	final public void clear()
	    {
		dbBase = new DbBase();
		frame.setFilename("Db filename: No filename");
		currentFilename = null;
		selectRecord.newInputFileNotify();
		menu.setFilename(null);
	}

	final public void openDbdFile(String fn)
	    {
		if (fn == null) return;
		try {
			dbBase.readDatabase(fn);
		}
		catch(Exception e) {
			new MessageDialog(frame,"Error","Error in reading dbd file "+fn);
			return;
		}
		selectRecord.newInputFileNotify();
	}

	final public void openDbFile(String fn)
	    {
		if (fn == null) return;
		try {
			dbBase.readDatabase(fn);
		}
		catch(Exception e) {
			new MessageDialog(frame,"Error","Error in reading database "+fn);
			return;
		}
		currentFilename = fn;
		frame.setFilename("Db filename: "+fn);
		menu.setFilename(fn);
		selectRecord.newInputFileNotify();
	}

	final private void saveFile(String fn)
	    {
		if (fn == null){
			new MessageDialog(frame,"Error","No db filename.  Use 'Save As'");
			return;
		}
		try {
			dbBase.writeRecord(fn);
			menu.setFilename(fn);
		}
		catch(Exception e) {
			new MessageDialog(frame,"Error","  from  Save file : writeRecord error: " + e);
		}
		currentFilename = fn;
		frame.setFilename("Db filename: "+fn);
		menu.setFilename(fn);
	}

	final private void saveAsFile(String fn)
	    {
		if (fn == null) return;
		try {
			dbBase.writeRecord(fn);
			menu.setFilename(fn);
		}
		catch(Exception e) {
			new MessageDialog(frame,"Error","  from  Save as : writeRecord error: " + e);
		}
		currentFilename = fn;
		frame.setFilename("Db filename: "+fn);
		menu.setFilename(fn);
	}

	final private void printFile()
	    {
		new MessageDialog(frame,"Comment","  Print not yet implemented");
	}

	public DbBase getDbBase()
	    {
		return dbBase;
	}

	public JdctRecordInstance getRecordInstance()
	    {
		return frame.getRecordInstance();
	}

	public void setRecordInstance(String typeName,String recordName)
	    {
		frame.setRecordInstance(typeName,recordName);
	}

	public void removeRecordInstance(JdctRecordInstance recordInstance)
	    {
		frame.removeRecordInstance(recordInstance);
	}

	final private void createRecord(String name, String type)
	    {
		DbBase dbBase = this.getDbBase();
		DbEntry dbEntry = new DbEntry(dbBase);
		if (dbEntry.findRecord(name)) {
			new MessageDialog(frame,"Error","  from  Create record : " + name +" already exists.");
		} else {
			if (dbEntry.findRecordType(type)) {
				dbEntry.createRecord(name);
				List list = frame.getInstanceList();
				list.add(name);
				int index=list.getItemCount() - 1;
				list.select(index);
			} else {
				new MessageDialog(frame,"Error","  from  Create record type : "+type+" not found.");
			}
		}
		menu.setRecordName(frame.getRecordName());
		getRecordInstance().loadInstance(frame.getRecordName());

	}

	final private void deleteRecord(String name)
	    {
		List list = frame.getInstanceList();
		int index = list.getSelectedIndex();
		DbBase dbBase = this.getDbBase();
		DbEntry dbEntry = new DbEntry(dbBase);
		if (dbEntry.findRecord(name)) {
			dbEntry.deleteRecord();
			list.remove(name);
		}
		if (list.getItemCount() > 0 ) {
			if (index < list.getItemCount() )
				list.select(index);
			else list.select(index-1);
		}
		menu.setRecordName(frame.getRecordName());
		getRecordInstance().loadInstance(frame.getRecordName());
	}

	final private void copyRecord(String fromName,String toName)
	    {
		if (fromName==null || toName==null) return;
		DbBase dbBase = this.getDbBase();
		DbEntry dbEntry = new DbEntry(dbBase);
		if (dbEntry.findRecord(fromName)) {
			try {
				dbEntry.copyRecord(toName,false);
			}
			catch (Exception e) {
				new MessageDialog(frame,"Error","  from  Copy record : " + toName +  "  already exists");
				return;
			}

			List list = frame.getInstanceList();
			list.add(toName);
			int index=list.getItemCount() - 1;
			list.select(index);

		}
		menu.setRecordName(frame.getRecordName());
		getRecordInstance().loadInstance(frame.getRecordName());
	}

	final private void renameRecord(String oldName,String newName)
	    {
		if (oldName==null || newName==null) return;
		DbBase dbBase = this.getDbBase();
		DbEntry dbEntry = new DbEntry(dbBase);
		if (dbEntry.findRecord(oldName)) {
			try {
				dbEntry.renameRecord(newName);
			}
			catch (Exception e)  {
				new MessageDialog(frame,"Error","  from Rename record : " + newName + " already exists");
				return;
			}
			List list = frame.getInstanceList();
			int index=list.getSelectedIndex();
			list.replaceItem(newName,index);
			list.select(index);
		}
		menu.setRecordName(frame.getRecordName());
		getRecordInstance().loadInstance(frame.getRecordName());
	}

	public void getRecordInstanceList()
	    {
		int nTypes = frame.getTypeList().getItemCount();
		if(nTypes<=0) return;
		String selectedTypes[] = frame.getTypeList().getSelectedItems();
		DbBase dbBase = this.getDbBase();
		DbEntry dbEntry = new DbEntry(dbBase);
		if(frame.getInstanceList().getItemCount()>0)
			frame.getInstanceList().removeAll();
		for(int indType=0; indType<selectedTypes.length; indType++) {
			if(!dbEntry.findRecordType(selectedTypes[indType])) continue;
			if(!dbEntry.firstRecord()) continue;
			do {
				frame.getInstanceList().add(dbEntry.getRecordName());
			} while (dbEntry.nextRecord());
		}
		dbEntry.free();
	}

	class MessageDialog extends Dialog
	    {
		public MessageDialog(Frame frame,String title,String message)
		    {
			super(frame,title,true);

			this.setLayout(new BorderLayout(15, 15));
			this.add("Center", new Label(message,Label.CENTER));

			Panel panel = new Panel();
			panel.setLayout(new FlowLayout(FlowLayout.CENTER, 25, 15));

			this.add("South", panel);

			Button okButton = new Button("Ok");
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					MessageDialog.this.dispose(); 																}
			}
			);

			this.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					((Dialog)e.getSource()).dispose(); 																}
			}
			);

			panel.add(okButton);

			this.pack();
			//this.setLocation(location);
			this.setVisible(true);
		}
	}
	class QuitDialog extends Dialog {
		public QuitDialog(Frame frame,String title,String message)
		    {
			super(frame,title,true);

			this.setLayout(new BorderLayout(15, 15));
			this.add("Center", new Label(message,Label.CENTER));

			Panel panel = new Panel();
			panel.setLayout(new FlowLayout(FlowLayout.CENTER, 25, 15));

			this.add("South", panel);

			Button okButton = new Button("Ok");
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					QuitDialog.this.dispose();
					    System.exit(0);
				}
			}
			);
			Button cancelButton = new Button("Cancel");
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					QuitDialog.this.dispose();
				}
			}
			);
			this.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					((Dialog)e.getSource()).dispose(); 									}
			}
			);
			panel.add(okButton);
			panel.add(cancelButton);
			this.pack();
			this.setVisible(true);
		}
	}
	public void windowClosing(WindowEvent e) {
		new QuitDialog(frame,"Quit Dialog","Quit Jdct?");
	}
	public void windowOpened(WindowEvent e) {
	}
	public void windowClosed(WindowEvent e) {
	}
	public void windowIconified(WindowEvent e) {
	}
	public void windowDeiconified(WindowEvent e) {
	}
	public void windowActivated(WindowEvent e) {
	}
	public void windowDeactivated(WindowEvent e) {
	}

}
