/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
package jdct;

import jdct.*;
import dbStatic.* ;
import java.lang.* ;
import java.awt.*;
import java.awt.List;
import java.awt.event.*;
import java.util.*;
import java.io.*;

class JdctMenu extends MenuBar
{
	private static final int TEXT_UPDATE_LEN = 30;
	private Menu file;
	private Menu edit;

	private String currentFilename;

	private DbBase dbBase = new DbBase();
	private JdctFrame frame;

	private String recordType= null;
	private String selectedRecordName= null;

	private CreateRecordDialog createRecordDialog;
	private DeleteRecordDialog deleteRecordDialog;
	private CopyRecordDialog copyRecordDialog;
	private RenameRecordDialog renameRecordDialog;

	static String filename = null;
	static String userDirectory = System.getProperty("user.dir");
	static String directory = System.getProperty("user.dir");
	static String fileSeparator = System.getProperty("file.separator");
	static String directoryDbd=".."+fileSeparator+".."+fileSeparator+"dbd";

	JdctMenu(JdctFrame frameIn) {
		super();
		MenuItem mi;

		frame = frameIn;
		this.getDbBase();

		file = new Menu("File");

		add(file);
		mi = new MenuItem("Open dbd");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String fn = jdctDbdFileDialog(frame,
				"Enter dbd filename", FileDialog.LOAD);
				openDbdFile(fn);
			} 
		}
		);
		file.add(mi);


		mi = new MenuItem("Open db");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String fn = jdctFileDialog(frame,
				    "Enter db filename", FileDialog.LOAD);
				openDbFile(fn);
			} 
		}
		);
		file.add(mi);

		file.addSeparator();
		mi = new MenuItem("Save");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveFile(currentFilename);
			} 
		}
		);
		file.add(mi);

		mi = new MenuItem("Save as");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String fn = jdctFileDialog(frame,
			   	 "Enter db filename", FileDialog.SAVE);
				saveAsFile(fn);
			} 
		}
		);
		file.add(mi);

		file.addSeparator();
		mi = new MenuItem("Clear");
		mi.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
				clear();
				frame.clearAllRecordInstances();
			} 
		}
		);
		file.add(mi);

		mi = new MenuItem("Clear db");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cleardb();
				frame.clearAllRecordInstances();
			} 
		}
		);
		file.add(mi);

		//file.addSeparator();
		//mi = new MenuItem("Print");
		//mi.addActionListener(new ActionListener() {
		//	public void actionPerformed(ActionEvent e) {
		//		printFile();
		//	} 
		//}
		//);
		//file.add(mi);

		file.addSeparator();
		mi = new MenuItem("Quit");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				frame.createQuitDialog();
			} 
		}
		);
		file.add(mi);

		edit = new Menu("Record");
		add(edit);
		mi = new MenuItem("Create");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (createRecordDialog == null ) {
					createRecordDialog =
						new CreateRecordDialog(frame);
				} else {
					createRecordDialog.dialog().toFront();
				}
			} 
		}
		);
		edit.add(mi);

		mi = new MenuItem("Delete");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (deleteRecordDialog == null ) {
					deleteRecordDialog =
						new DeleteRecordDialog(frame,
						selectedRecordName);
				} else {
					deleteRecordDialog.dialog().toFront();
				}
			} 
		}
		);
		edit.add(mi);

		mi = new MenuItem("Copy");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (copyRecordDialog == null ) {
					copyRecordDialog = 
						new CopyRecordDialog(frame,
						selectedRecordName);
				} else {
					copyRecordDialog.dialog().toFront();
				}
			}
		}
		);
		edit.add(mi);

		mi = new MenuItem("Rename");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (renameRecordDialog == null ) {
					renameRecordDialog =
						new RenameRecordDialog(frame,
						selectedRecordName);
				} else {
					renameRecordDialog.dialog().toFront();
				}
			} 
		}
		);
		edit.add(mi);
	}

	public DbBase getDbBase()
	    {
		return dbBase;
	}

	public void setFilename(String filename) {
		this.filename = filename;
	}

	public void setSelectedRecordType(String recordType) {
		if (createRecordDialog != null){
			createRecordDialog.getRecordType();
		}
	}

	public String getFilename() {
		return filename;
	}

	public String jdctDbdFileDialog(Frame frameIn,String title,int type) {
		FileDialog dialog = new FileDialog(frameIn,title,type);
		if (!new File(directoryDbd).isDirectory()){
			directoryDbd = System.getProperty("user.dir");
		}
		dialog.setDirectory(directoryDbd);
		SuffixFilter filter = new SuffixFilter("dbd");
		dialog.setFilenameFilter(filter);
		dialog.setVisible(true);

		String fn = dialog.getFile();
		if (fn==null) return fn;
		directoryDbd = dialog.getDirectory();
		if (directoryDbd!=null && !directoryDbd.substring(0,
		    directoryDbd.length()-1).equals(userDirectory)) {
			fn = directoryDbd + fn;
		}
		return fn;
	}

	public String jdctFileDialog(Frame frameIn,String title,int type) {
		FileDialog dialog = new FileDialog(frameIn,title,type);
		dialog.setDirectory(directory);
		if (filename != null ) dialog.setFile(filename);
		SuffixFilter filter = new SuffixFilter("db");
		dialog.setFilenameFilter(filter);
		dialog.setVisible(true);

		String fn = dialog.getFile();
		if (fn==null) return fn;
		directory = dialog.getDirectory();
		if (directory!=null && 
		    !directory.substring(0,directory.length()-1).equals(userDirectory)) {
			fn = directory + fn;
		}
		return fn;
	}

	public void setRecordName(String text) {
		selectedRecordName = text;
		if (deleteRecordDialog != null){
			deleteRecordDialog.setName(text);
		}
		if (copyRecordDialog != null){
			copyRecordDialog.setName(text);
		}
		if (renameRecordDialog != null){
			renameRecordDialog.setName(text);
		}
	}

	public String getCreateRecordType()
	{
		if (createRecordDialog != null){
			return createRecordDialog.getRecordType();
		}
		else return null;
	}

	String getRecordName()
	{
		return selectedRecordName;
	}

	String getCreateName()
	{
		if (createRecordDialog != null){
			return createRecordDialog.getNewName();
		} 
		else return null;
	}

	String getRenameName()
	{
		if (renameRecordDialog != null){
			return renameRecordDialog.getNewName();
		}
		else return null;
	}

	String getCopyName()
	{
		if (copyRecordDialog != null){
			return copyRecordDialog.getNewName();
		}
		else return null;
	}


	final public void openDbdFile(String fn)
	{
		if (fn == null) return;
		try {
			dbBase.readDatabase(fn);
		}
		catch(Exception e) {
			new MessageDialog(frame,
			"Error","Error in reading dbd file "+fn);
			return;
		}
		newInputFileNotify();
		if (createRecordDialog != null){
			createRecordDialog.updateTypeMenu();
		}
	}

	final public void openDbFile(String fn)
	{
		if (fn == null) return;
		try {
			dbBase.readDatabase(fn);
		}
		catch(Exception e) {
			new MessageDialog(frame,
			"Error","Error in reading database "+fn);
			return;
		}
		currentFilename = fn;
		frame.setFilename("Database filename: "+fn);
		setFilename(fn);
		newInputFileNotify();
	}

	final public void clear()
	{
		dbBase = new DbBase();
		frame.setFilename("Database filename: No filename");
		currentFilename = null;
		setFilename(null);
		setRecordName(null);
		newInputFileNotify();
		if (createRecordDialog != null){
			createRecordDialog.updateTypeMenu();
		}
	}

	final public void cleardb()
	    {
		frame.setFilename("Database filename: No filename");
		currentFilename = null;
		setFilename(null);
		setRecordName(null);
		newInputFileNotify();

		DbEntry dbEntry = new DbEntry(dbBase);
		if(!dbEntry.firstRecordType()) return;
		do {
			while (dbEntry.firstRecord()) {
				dbEntry.deleteRecord();
			}
		} while (dbEntry.nextRecordType());
	}

	final private void saveFile(String fn)
	{
		if (fn == null){
			new MessageDialog(frame,
			"Error","No db filename.  Use 'Save As'");
			return;
		}
		try {
			dbBase.writeRecord(fn);
			setFilename(fn);
		}
		catch(Exception e) {
			new MessageDialog(frame,
			"Error","  from  Save file : writeRecord error: " + e);
		}
		currentFilename = fn;
		frame.setFilename("Database filename: "+fn);
		setFilename(fn);
	}

	final private void saveAsFile(String fn)
	{
		if (fn == null) return;
		try {
			dbBase.writeRecord(fn);
			setFilename(fn);
		}
		catch(Exception e) {
			new MessageDialog(frame,
			"Error","  from  Save as : writeRecord error: " + e);
		}
		currentFilename = fn;
		frame.setFilename("Database filename: "+fn);
		setFilename(fn);
	}

	final private void printFile()
	{
		new MessageDialog(frame,
		"Comment","  Print not yet implemented");
	}

	final private void createRecord(String name, String type)
	{
		DbEntry dbEntry = new DbEntry(dbBase);
		if (dbEntry.findRecord(name)) {
			new MessageDialog(frame,
			"Error","  from  Create record : " +
			name +" already exists.");
			return;
		}
		if (type==null) {
			new MessageDialog(frame,
			"Error","  from  Create record : "+
			" No record types.");
			return;
		}
		if (dbEntry.findRecordType(type)) {
			dbEntry.createRecord(name);
			List list = frame.getInstanceList();
			list.add(name);
			int index=list.getItemCount() - 1;
			list.select(index);
		} else {
			new MessageDialog(frame,
			"Error","  from  Create record : "+
			type+" not found.");
			return;
		}
		setRecordName(frame.getRecordName());
		JdctRecordInstance recordInstance = frame.getRecordInstance();
		if (recordInstance == null ) {
			frame.addRecordInstance();
			recordInstance = frame.getRecordInstance();
		}
		recordInstance.loadInstance(frame.getRecordName());

	}

	final private void deleteRecord(String name)
	{
		List list = frame.getInstanceList();
		int index = list.getSelectedIndex();
		DbEntry dbEntry = new DbEntry(dbBase);
		if (dbEntry.findRecord(name)) {
			dbEntry.deleteRecord();
			list.remove(name);
		}
		if (list.getItemCount() > 0 ) {
			if (index < list.getItemCount() ) list.select(index);
			else list.select(index-1);
		}
		setRecordName(frame.getRecordName());
		Vector instanceVector = frame.getInstanceVector();
		JdctRecordInstance instance;
		for (int i=0; i<instanceVector.size(); i++) {
			instance = (JdctRecordInstance)(instanceVector.elementAt(i));
		        if ( instance.getRecordName().equals(name)) {
				instance.getInstanceGUI().setVisible(false);
				instance.getInstanceGUI().dispose();
				frame.removeRecordInstance(instance);
			}
		}
		JdctRecordInstance recordInstance = frame.getRecordInstance();
		if (recordInstance != null ) {
			frame.setRecordInstance(recordInstance.getTypeName(),
				recordInstance.getRecordName());
			recordInstance.loadInstance(frame.getRecordName());
		}
	}

	final private void copyRecord(String fromName,String toName)
	{
		if (fromName==null || toName==null) return;
		DbEntry dbEntry = new DbEntry(dbBase);
		if (dbEntry.findRecord(fromName)) {
			try {
				dbEntry.copyRecord(toName,false);
			}
			catch (Exception e) {
				new MessageDialog(frame,
				"Error","  from  Copy record : " +
				toName +  "  already exists");
				return;
			}

			List list = frame.getInstanceList();
			list.add(toName);
			int index=list.getItemCount() - 1;
			list.select(index);

		}
		setRecordName(frame.getRecordName());
		JdctRecordInstance recordInstance = frame.getRecordInstance();
		if (recordInstance != null ) {
			recordInstance.loadInstance(frame.getRecordName());
		}
	}

	final private void renameRecord(String oldName,String newName)
	    {
		if (oldName==null || newName==null) return;
		DbEntry dbEntry = new DbEntry(dbBase);
		if (dbEntry.findRecord(oldName)) {
			try {
				dbEntry.renameRecord(newName);
			}
			catch (Exception e)  {
				new MessageDialog(frame,
				"Error","  from Rename record : " + 
				newName + " already exists");
				return;
			}
			List list = frame.getInstanceList();
			int index=list.getSelectedIndex();
			list.replaceItem(newName,index);
			list.select(index);
		}
		setRecordName(frame.getRecordName());
		JdctRecordInstance recordInstance = frame.getRecordInstance();
		if (recordInstance != null ) {
			recordInstance.loadInstance(frame.getRecordName());
		}
	}

	class CreateRecordDialog
	{
		JdctFrame jdctFrame;
		Dialog dialog;
		TextField text1;
		Choice choice;

		CreateRecordDialog(JdctFrame jdctFrame)
		    {
			this.jdctFrame = jdctFrame;

			dialog = new Dialog(jdctFrame,"Create record",false);

			Panel entryPanel = new Panel();
			entryPanel.setLayout(new GridLayout(0,1));

			entryPanel.add(new Label("Record name: "));
			text1 = new TextField("",JdctMenu.TEXT_UPDATE_LEN);
			entryPanel.add(text1);
			choice = new Choice();
			for (int i=0; i<jdctFrame.getTypeList().getItemCount(); i++)
				choice.add(jdctFrame.getTypeList().getItem(i));
			choice.select(0);
			entryPanel.add(choice);

			dialog.add("North", entryPanel);

			Panel panel = new Panel();
			panel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
			Button okButton = new Button("Ok");
			okButton.setActionCommand("Create");
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
				String type = getCreateRecordType();
				String name = getCreateName();
				if (name!=null && name.length()>0) {
					createRecord(name,type);
				} 
			} 
			}
			);
			panel.add(okButton);

			Button dismissButton = new Button("Dismiss");
			dismissButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					dialog.setVisible(false);
					dialog.dispose();
					createRecordDialog = null;
			} 
			}
			);
			panel.add(dismissButton);
			dialog.add("South", panel);

			dialog.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					dialog.dispose();
					createRecordDialog = null;
				}
			}
			);

			dialog.pack();
			dialog.setVisible(true);
		}

		public Dialog dialog() {
			return	dialog;
		}

		public String getRecordType() {
			return	choice.getSelectedItem();
		}

		public String getNewName() {
			return text1.getText();
		}

		public void updateTypeMenu() {
			choice.removeAll();
			for (int i=0; i<jdctFrame.getTypeList().getItemCount(); i++)
				choice.add(jdctFrame.getTypeList().getItem(i));
		}
	}

	class DeleteRecordDialog
	{
		Dialog dialog;
		TextField text1;

		DeleteRecordDialog(Frame frameIn,String name)
		    {

			dialog = new Dialog(frameIn,"Delete record",false);

			Panel entryPanel = new Panel();
			entryPanel.setLayout(new GridLayout(0,1));

			entryPanel.add(new Label("Record name: "));
			text1 = new TextField("",JdctMenu.TEXT_UPDATE_LEN);
			text1.setText(name);
			text1.setEditable(false);
			entryPanel.add(text1);

			dialog.add("North", entryPanel);

			Panel panel = new Panel();
			panel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
			Button okButton = new Button("Ok");
			okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String recName = getRecordName();
				if (recName!=null) {
					deleteRecord(recName);
				} 
			} 
			}
			);
			panel.add(okButton);

			Button dismissButton = new Button("Dismiss");
			dismissButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if ( e.getActionCommand()=="Dismiss") {
						dialog.setVisible(false);
						dialog.dispose();
						deleteRecordDialog = null;
					}
				}

			} 
			);
			panel.add(dismissButton);

			dialog.add("South", panel);
			dialog.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					dialog.dispose();
					deleteRecordDialog = null;
				}
			}
			);

			dialog.pack();
			dialog.setVisible(true);
		}

		public Dialog dialog() {
			return	dialog;
		}

		public String getName() {
			return text1.getText();
		}

		public void setName(String text) {
			text1.setText(text);
		}
	}


	class CopyRecordDialog 
	{

		Dialog dialog;
		TextField text1;
		TextField text2;

		CopyRecordDialog(Frame frameIn,String name)
		    {
			dialog = new Dialog(frameIn,"Copy record",false);

			Panel entryPanel = new Panel();
			entryPanel.setLayout(new GridLayout(0,1));

			entryPanel.add(new Label("From record name: "));
			text1 = new TextField("",JdctMenu.TEXT_UPDATE_LEN);
			text1.setText(name);
			text1.setEditable(false);
			entryPanel.add(text1);
			entryPanel.add(new Label("To record name: "));
			text2 = new TextField("",JdctMenu.TEXT_UPDATE_LEN);
			entryPanel.add(text2);

			dialog.add("North", entryPanel);

			Panel panel = new Panel();
			panel.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
			Button okButton = new Button("Ok");
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String recName = JdctMenu.this.getRecordName();
					String newName = JdctMenu.this.getCopyName();
					if (newName!=null && newName.length()>0) {
						copyRecord(recName,newName);
					}
				} 
			}
			);

			panel.add(okButton);
			Button dismissButton = new Button("Dismiss");
			dismissButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					dialog.setVisible(false);
					dialog.dispose();
					copyRecordDialog = null;
				}
			} 
			);
			panel.add(dismissButton);
			dialog.add("South", panel);

			dialog.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					dialog.dispose();
					copyRecordDialog = null;
				}
			}
			);

			dialog.pack();
			dialog.setVisible(true);
		}

		public Dialog dialog() {
			return	dialog;
		}

		public String getNewName() {
			return text2.getText();
		}

		public void setName(String text) {
			text1.setText(text);
		}
	}


	class RenameRecordDialog 
	{

		Dialog dialog;
		TextField text1;
		TextField text2;

		RenameRecordDialog(Frame frameIn,String name)
		    {
			dialog = new Dialog(frameIn,"Rename record",false);

			Panel entryPanel = new Panel();
			entryPanel.setLayout(new GridLayout(0,1));

			entryPanel.add(new Label("Old record name: "));
			text1 = new TextField("",JdctMenu.TEXT_UPDATE_LEN);
			text1.setText(name);
			text1.setEditable(false);
			entryPanel.add(text1);
			entryPanel.add(new Label("New record name: "));
			text2 = new TextField("",JdctMenu.TEXT_UPDATE_LEN);
			entryPanel.add(text2);

			dialog.add("North", entryPanel);

			Panel panel = new Panel();
			panel.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
			Button okButton = new Button("Ok");
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String recName = JdctMenu.this.getRecordName();
					String newName = JdctMenu.this.getRenameName();
					if (newName!=null && newName.length()>0){
						renameRecord(recName,newName);
					}
				} 
		}
		);

			panel.add(okButton);
			Button dismissButton = new Button("Dismiss");
			dismissButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					dialog.setVisible(false);
					dialog.dispose();
					renameRecordDialog = null;
				}
			} 
			);
			panel.add(dismissButton);
			dialog.add("South", panel);

			dialog.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					dialog.dispose();
					renameRecordDialog = null;
				}
			}
			);

			dialog.pack();
			dialog.setVisible(true);
		}

		public Dialog dialog() {
			return	dialog;
		}

		public String getNewName() {
			return text2.getText();
		}

		public void setName(String text) {
			text1.setText(text);
		}
	}

	class MessageDialog extends Dialog
	{
		public MessageDialog(Frame frameIn,String title,String message)
		    {
			super(frameIn,title,true);

			this.setLayout(new BorderLayout(15, 15));
			this.add("Center", new Label(message,Label.CENTER));

			Panel panel = new Panel();
			panel.setLayout(new FlowLayout(FlowLayout.CENTER, 25, 15));

			this.add("South", panel);

			Button okButton = new Button("Ok");
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					MessageDialog.this.dispose();
					}
			}
			);

			this.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					((Dialog)e.getSource()).dispose(); 																}
			}
			);

			panel.add(okButton);

			this.pack();
			//this.setLocation(location);
			this.setVisible(true);
		}
	}

	class SuffixFilter implements FilenameFilter
	{
		private String suffix;

		public SuffixFilter(String suffix) { 
			this.suffix = suffix; 
		}

		public boolean accept(File dir, String name) {
			//if (dir.isDirectory()) return true;
			return name.toLowerCase().endsWith("."+suffix);
		}
		public String getDescription() {
			return suffix + " file (*." +suffix + ")";
		}

	}

	final void newInputFileNotify() {
		DbEntry dbEntry = new DbEntry(dbBase);
		String recordTypename;

		if(frame.getTypeList().getItemCount()>0)
			frame.getTypeList().removeAll();
		if(!dbEntry.firstRecordType()) {
			// System.out.println("No record types");
		} else do {
			recordTypename = dbEntry.getRecordTypeName();
			frame.getTypeList().add(recordTypename);
		} while (dbEntry.nextRecordType());
		frame.pack();
		dbEntry.free();
	}
}
